"""
URL configuration for the ``rest_email_auth`` app.
"""

from django.conf.urls import url

from rest_email_auth import views


app_name = "rest-email-auth"


urlpatterns = [
    url(r"^emails/$", views.EmailListView.as_view(), name="email-list"),
    url(
        r"^emails/(?P<pk>[0-9]+)/$",
        views.EmailDetailView.as_view(),
        name="email-detail",
    ),
    url(r"^register/$", views.RegistrationView.as_view(), name="register"),
    url(
        r"^request-password-reset/$",
        views.PasswordResetRequestView.as_view(),
        name="password-reset-request",
    ),
    url(
        r"^resend-verification/$",
        views.ResendVerificationView.as_view(),
        name="resend-verification",
    ),
    url(
        r"^reset-password/$",
        views.PasswordResetView.as_view(),
        name="password-reset",
    ),
    url(
        r"^verify-email/$",
        views.EmailVerificationView.as_view(),
        name="verify-email",
    ),
]



# Generated by Django 2.2.1 on 2019-05-03 09:44

from django.db import migrations
from django.db.models import Count


def fix_primary_emails(apps, schema_editor):
    """
    For all users who only have a single email address, set it as their
    primary.
    """
    EmailAddress = apps.get_model("rest_email_auth", "EmailAddress")

    EmailAddress.objects.annotate(count=Count("user__email_address")).filter(
        count=1
    ).update(is_primary=True)


class Migration(migrations.Migration):

    dependencies = [("rest_email_auth", "0005_passwordresettoken")]

    operations = [
        migrations.RunPython(
            fix_primary_emails, reverse_code=migrations.RunPython.noop
        )
    ]



"""
Tests for the verified email backend.
"""


def test_authenticate(
    email_factory, user_factory, verified_email_auth_backend
):
    """
    Passing in a verified email address and the user's password should
    successfully authenticate the user.
    """
    user = user_factory(password="password")
    email = email_factory(is_verified=True, user=user)

    assert (
        verified_email_auth_backend.authenticate(
            email=email.email, password="password", request=None
        )
        == user
    )


def test_authenticate_invalid_password(
    email_factory, user_factory, verified_email_auth_backend
):
    """
    Providing the wrong password should cause ``None`` to be returned.
    """
    user = user_factory(password="password")
    email = email_factory(is_verified=True, user=user)

    assert (
        verified_email_auth_backend.authenticate(
            email=email.email, password="notpassword", request=None
        )
        is None
    )


def test_authenticate_missing_email(user_factory, verified_email_auth_backend):
    """
    If the provided email doesn't exist in the system, ``None`` should
    be returned.
    """
    user_factory(password="password")

    assert (
        verified_email_auth_backend.authenticate(
            email="missing@example.com", password="password", request=None
        )
        is None
    )


def test_authenticate_unverified_email(
    email_factory, user_factory, verified_email_auth_backend
):
    """
    If the provided email address exists but has not been verified yet
    ``None`` should be returned.
    """
    user = user_factory(password="password")
    email = email_factory(is_verified=False, user=user)

    assert (
        verified_email_auth_backend.authenticate(
            email=email.email, password="password", request=None
        )
        is None
    )


def test_authenticate_username(
    email_factory, user_factory, verified_email_auth_backend
):
    """
    The ``username`` parameter should act as an alias for the ``email``
    parameter.
    """
    user = user_factory(password="password")
    email = email_factory(is_verified=True, user=user)

    assert (
        verified_email_auth_backend.authenticate(
            password="password", request=None, username=email.email
        )
        == user
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-25 13:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("rest_email_auth", "0002_emailconfirmation")]

    operations = [
        migrations.AlterField(
            model_name="emailaddress",
            name="email",
            field=models.EmailField(
                max_length=255, unique=True, verbose_name="email"
            ),
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-02-05 13:53
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [("rest_email_auth", "0004_emailaddress_is_primary")]

    operations = [
        migrations.CreateModel(
            name="PasswordResetToken",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time at which the password reset token was created.",
                        verbose_name="created at",
                    ),
                ),
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        help_text="The token that authorizes the user to reset their password.",
                        verbose_name="key",
                    ),
                ),
                (
                    "email",
                    models.ForeignKey(
                        editable=False,
                        help_text="The email address used to request the password reset.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="password_reset_tokens",
                        related_query_name="password_reset_token",
                        to="rest_email_auth.EmailAddress",
                        verbose_name="email address",
                    ),
                ),
            ],
            options={
                "verbose_name": "password reset token",
                "verbose_name_plural": "password reset tokens",
            },
        )
    ]



try:
    from unittest import mock
except ImportError:
    import mock

from rest_email_auth import serializers


def test_confirm_email(
    email_confirmation_factory, email_factory, user_factory
):
    """
    If a valid confirmation key and password are provided, the email
    matching the key should be verified.
    """
    user = user_factory(password="password")
    email = email_factory(user=user)
    confirmation = email_confirmation_factory(email=email)

    data = {"key": confirmation.key, "password": "password"}

    serializer = serializers.EmailVerificationSerializer(data=data)
    assert serializer.is_valid()

    serializer.save()
    email.refresh_from_db()

    expected = {"email": email.email}

    assert serializer.data == expected
    assert email.is_verified
    assert email.confirmations.count() == 0


def test_validate_expired(email_confirmation_factory):
    """
    If the confirmation matching the provided key is expired, the
    serializer should not be valid.
    """
    confirmation = email_confirmation_factory()

    data = {"key": confirmation.key, "password": "password"}

    serializer = serializers.EmailVerificationSerializer(data=data)

    with mock.patch(
        "rest_email_auth.serializers.models.EmailConfirmation.is_expired",
        new_callable=mock.PropertyMock,
    ) as mock_is_expired:
        mock_is_expired.return_value = True

        assert not serializer.is_valid()

    assert mock_is_expired.call_count == 1
    assert set(serializer.errors.keys()) == {"key"}


def test_validate_invalid_key(db):
    """
    If there is no confirmation with the provided key, the serializer
    should not be valid.
    """
    data = {"key": "invalid", "password": "password"}

    serializer = serializers.EmailVerificationSerializer(data=data)

    assert not serializer.is_valid()
    assert set(serializer.errors.keys()) == {"key"}


def test_validate_invalid_password(
    email_confirmation_factory, email_factory, user_factory
):
    """
    If an invalid password is provided, the serializer should not be
    valid.
    """
    user = user_factory(password="password")
    email = email_factory(user=user)
    confirmation = email_confirmation_factory(email=email)

    data = {"key": confirmation.key, "password": "notpassword"}

    serializer = serializers.EmailVerificationSerializer(data=data)

    assert not serializer.is_valid()
    assert set(serializer.errors.keys()) == {"non_field_errors"}


def test_validate_no_password(
    app_settings, email_confirmation_factory, email_factory, user_factory
):
    """
    If the password requirement for verifying email addresses is
    disabled then users should be able to verify email addresses without
    providing their password.
    """
    app_settings.EMAIL_VERIFICATION_PASSWORD_REQUIRED = False

    user = user_factory()
    email = email_factory(user=user)
    confirmation = email_confirmation_factory(email=email)

    data = {"key": confirmation.key}

    serializer = serializers.EmailVerificationSerializer(data=data)
    assert serializer.is_valid()

    serializer.save()
    email.refresh_from_db()

    expected = {"email": email.email}

    assert serializer.data == expected
    assert email.is_verified
    assert email.confirmations.count() == 0



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-31 23:58
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="EmailAddress",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="created at"
                    ),
                ),
                (
                    "email",
                    models.EmailField(max_length=255, verbose_name="email"),
                ),
                (
                    "is_verified",
                    models.BooleanField(
                        default=False,
                        help_text="Boolean indicating if the user has verified ownership of the email address.",
                        verbose_name="is verified",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="email_addresses",
                        related_query_name="email_address",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "email address",
                "verbose_name_plural": "email addresses",
            },
        )
    ]



try:
    from unittest import mock
except ImportError:
    import mock

from rest_framework import status
from rest_framework.reverse import reverse


url = reverse("rest-email-auth:password-reset")


@mock.patch(
    "rest_email_auth.views.serializers.PasswordResetSerializer.save",
    autospec=True,
)
def test_request_reset(mock_save, api_client, password_reset_token_factory):
    """
    Sending a POST request with valid data to the view should reset the
    user's password.
    """
    token = password_reset_token_factory()
    data = {"key": token.key, "password": "new_passw0rd"}

    response = api_client.post(url, data)

    assert response.status_code == status.HTTP_200_OK
    assert mock_save.call_count == 1



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-08 15:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import rest_email_auth.models


class Migration(migrations.Migration):

    dependencies = [("rest_email_auth", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="EmailConfirmation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="created at"
                    ),
                ),
                (
                    "key",
                    models.CharField(
                        default=rest_email_auth.models.generate_token,
                        editable=False,
                        max_length=255,
                        verbose_name="confirmation key",
                    ),
                ),
                (
                    "email",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="confirmations",
                        related_query_name="confirmation",
                        to="rest_email_auth.EmailAddress",
                        verbose_name="email",
                    ),
                ),
            ],
            options={
                "verbose_name": "email confirmation",
                "verbose_name_plural": "email confirmations",
            },
        )
    ]



try:
    from unittest import mock
except ImportError:
    import mock

from django.contrib.auth import get_user_model

import pytest

from rest_email_auth import serializers


@pytest.mark.django_db
def test_create_new_user(registration_listener):
    """
    Saving a serializer with valid data should create a new user.
    """
    data = {
        "email": "test@example.com",
        "password": "password",
        "username": "user",
    }

    serializer = serializers.RegistrationSerializer(data=data)
    assert serializer.is_valid()

    with mock.patch(
        "rest_email_auth.serializers.models.EmailAddress.send_confirmation",  # noqa
        autospec=True,
    ) as mock_send_confirmation:
        user = serializer.save()

    assert user.username == data["username"]
    assert user.check_password(data["password"])
    assert user.email_addresses.count() == 1

    email = user.email_addresses.get()

    assert email.email == data["email"]
    assert email.is_primary

    # Make sure we sent out an email confirmation
    assert mock_send_confirmation.call_count == 1

    # Make sure we sent a registration event
    assert registration_listener.call_count == 1


def test_register_duplicate_email(email_factory, registration_listener):
    """
    Attempting to register with an email that has already been verified
    should send an email to the owner of the email address notifying
    them of the registration attempt.
    """
    email = email_factory()

    data = {"email": email.email, "password": "password", "username": "user"}

    serializer = serializers.RegistrationSerializer(data=data)
    assert serializer.is_valid()

    with mock.patch(
        "rest_email_auth.serializers.models.EmailAddress.send_duplicate_notification",  # noqa
        autospec=True,
    ) as mock_send_duplicate_signup:
        serializer.save()

    assert get_user_model().objects.count() == 1
    assert mock_send_duplicate_signup.call_count == 1

    # We should not get a registration event for a duplicate email
    assert registration_listener.call_count == 0


def test_validate_email_lowercase_domain():
    """
    The registration serializer should not change an email address with
    a lowercase domain.
    """
    email = "Test@example.com"
    serializer = serializers.RegistrationSerializer()

    assert serializer.validate_email(email) == email


def test_validate_email_mixed_case_domain():
    """
    If the domain portion of the email is mixed case, it should be
    converted to lowercase.
    """
    email = "Test@ExaMple.com"
    expected = "Test@example.com"
    serializer = serializers.RegistrationSerializer()

    assert serializer.validate_email(email) == expected


@pytest.mark.django_db
def test_validate_password():
    """
    Validating the serializer's data should run the provided password
    through Django's default password validation system.
    """
    data = {
        "email": "test@example.com",
        "password": "password",
        "username": "user",
    }

    serializer = serializers.RegistrationSerializer(data=data)

    with mock.patch(
        "rest_email_auth.serializers.password_validation.validate_password",  # noqa
        autospec=True,
    ) as mock_validate:
        assert serializer.is_valid()

    assert mock_validate.call_count == 1
    assert set(mock_validate.call_args[0]) == {data["password"]}



try:
    from unittest import mock
except ImportError:
    import mock

from django.conf import settings

from rest_email_auth import models


def test_create(user_factory):
    """
    Test creating an email address.
    """
    email = models.EmailAddress.objects.create(
        email="test@example.com", is_primary=True, user=user_factory()
    )

    assert not email.is_verified


def test_create_second_primary(email_factory):
    """
    If the user has a primary email address and a second one is created
    with ``is_primary == True``, the initial email should be removed as
    the primary email.
    """
    old_primary = email_factory(is_primary=True)
    email_factory(is_primary=True, user=old_primary.user)

    old_primary.refresh_from_db()

    assert not old_primary.is_primary


def test_send_confirmation(email_factory):
    """
    Sending a confirmation should create an ``EmailConfirmation``
    instance and send a verification email.
    """
    email = email_factory()

    with mock.patch(
        "rest_email_auth.models.EmailConfirmation.send", autospec=True
    ) as mock_send:
        email.send_confirmation()

    assert email.confirmations.count() == 1
    assert mock_send.call_count == 1


@mock.patch("rest_email_auth.models.email_utils.send_email")
def test_send_duplicate_notification(mock_send_email, email_factory):
    """
    Sending a duplicate signup notification should send the user an
    email stating that their email was used to register even though
    they already have an account.
    """
    email = email_factory()
    email.send_duplicate_notification()

    assert mock_send_email.call_count == 1
    assert mock_send_email.call_args[1] == {
        "from_email": settings.DEFAULT_FROM_EMAIL,
        "recipient_list": [email.email],
        "subject": "Registration Attempt",
        "template_name": "rest_email_auth/emails/duplicate-email",
    }


def test_set_primary(email_factory):
    """
    Setting an email as the primary should update all of the user's
    other email addresses to not be the primary.
    """
    old = email_factory(is_primary=True)
    new = email_factory(is_primary=False, user=old.user)

    new.set_primary()
    old.refresh_from_db()

    assert new.is_primary
    assert not old.is_primary


def test_string_conversion(email_factory):
    """
    Converting an email address to a string should return the email
    address.
    """
    email = email_factory()

    assert str(email) == email.email



"""
Custom signals used by django-rest-email-auth.

These signals allow other applications to easily add custom behavior to
processes from this module.
"""

from django import dispatch


email_verified = dispatch.Signal(providing_args=["email"])

user_registered = dispatch.Signal(providing_args=["user"])



from setuptools import setup, find_packages


def get_description():
    with open("README.rst") as f:
        return f.read()


setup(
    # Package meta-data
    name="django-rest-email-auth",
    version="2.0.3",
    description="Django app for email based authentication and registration.",
    long_description=get_description(),
    author="Chathan Driehuys",
    author_email="cdriehuys@gmail.com",
    url="https://github.com/cdriehuys/django-rest-email-auth",
    license="MIT",
    # Additional classifiers for PyPI
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        # Supported versions of Django
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        # Supported versions of Python
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    # Include the actual source code
    include_package_data=True,
    packages=find_packages(),
    # Dependencies
    install_requires=[
        "Django >= 1.10",
        "django-email-utils < 1.0",
        "djangorestframework >= 3.0",
    ],
)



from django.db import models, transaction
from django.utils import timezone

from rest_email_auth import app_settings


class EmailAddressManager(models.Manager):
    """
    Manager for email address instances.
    """

    def create(self, *args, **kwargs):
        """
        Create a new email address.
        """
        is_primary = kwargs.pop("is_primary", False)

        with transaction.atomic():
            email = super(EmailAddressManager, self).create(*args, **kwargs)

            if is_primary:
                email.set_primary()

        return email


class ValidPasswordResetTokenManager(models.Manager):
    """
    Manager for getting only valid password reset tokens.

    Valid tokens are those that have not yet expired.
    """

    def get_queryset(self):
        """
        Return all unexpired password reset tokens.
        """
        oldest = timezone.now() - app_settings.PASSWORD_RESET_EXPIRATION
        queryset = super(ValidPasswordResetTokenManager, self).get_queryset()

        return queryset.filter(created_at__gt=oldest)



"""Serializers for the ``rest_email_auth`` app.

The serializers handle the conversion of data between the JSON or form
data the API receives and native Python datatypes.
"""

import logging

from django.contrib.auth import get_user_model, password_validation
from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers

from rest_email_auth import models, signals, app_settings

logger = logging.getLogger(__name__)


class EmailSerializer(serializers.ModelSerializer):
    """
    Serializer for email addresses.
    """

    class Meta(object):
        extra_kwargs = {
            "email": {
                # We remove the autogenerated 'unique' validator to
                # avoid leaking email addresses.
                "validators": []
            }
        }
        fields = ("id", "created_at", "email", "is_primary", "is_verified")
        model = models.EmailAddress
        read_only_fields = ("is_verified",)

    def create(self, validated_data):
        """
        Create a new email and send a confirmation to it.

        Returns:
            The newly creating ``EmailAddress`` instance.
        """
        email_query = models.EmailAddress.objects.filter(
            email=self.validated_data["email"]
        )

        if email_query.exists():
            email = email_query.get()

            email.send_duplicate_notification()
        else:
            email = super(EmailSerializer, self).create(validated_data)
            email.send_confirmation()

            user = validated_data.get("user")
            query = models.EmailAddress.objects.filter(
                is_primary=True, user=user
            )

            if not query.exists():
                email.set_primary()

        return email

    def update(self, instance, validated_data):
        """
        Update the instance the serializer is bound to.

        Args:
            instance:
                The instance the serializer is bound to.
            validated_data:
                The data to update the serializer with.

        Returns:
            The updated instance.
        """
        is_primary = validated_data.pop("is_primary", False)

        instance = super(EmailSerializer, self).update(
            instance, validated_data
        )

        if is_primary:
            instance.set_primary()

        return instance

    def validate_email(self, email):
        """
        Validate the provided email address.

        The email address is first modified to match the RFC spec.
        Namely, the domain portion of the email is lowercased.

        Returns:
            The validated email address.

        Raises:
            serializers.ValidationError:
                If the serializer is bound and the provided email
                doesn't match the existing address.
        """
        user, domain = email.rsplit("@", 1)
        email = "@".join([user, domain.lower()])

        if self.instance and email and self.instance.email != email:
            raise serializers.ValidationError(
                _(
                    "Existing emails may not be edited. Create a new one "
                    "instead."
                )
            )

        return email

    def validate_is_primary(self, is_primary):
        """
        Validate the provided 'is_primary' parameter.

        Returns:
            The validated 'is_primary' value.

        Raises:
            serializers.ValidationError:
                If the user attempted to mark an unverified email as
                their primary email address.
        """
        # TODO: Setting 'is_primary' to 'False' should probably not be
        #       allowed.
        if is_primary and not (self.instance and self.instance.is_verified):
            raise serializers.ValidationError(
                _(
                    "Unverified email addresses may not be used as the "
                    "primary address."
                )
            )

        return is_primary


class EmailVerificationSerializer(serializers.Serializer):
    """
    Serializer for verifying an email address.
    """

    email = serializers.EmailField(read_only=True)
    key = serializers.CharField(write_only=True)
    password = serializers.CharField(
        style={"input_type": "password"}, write_only=True
    )

    def __init__(self, *args, **kwargs):
        """
        Conditionally remove the password field based on if a password
        is required to verify an email address.
        """
        super().__init__(*args, **kwargs)

        self._confirmation = None

        if not app_settings.EMAIL_VERIFICATION_PASSWORD_REQUIRED:
            self.fields.pop("password")

    def save(self):
        """
        Confirm the email address matching the confirmation key.
        """
        self._confirmation.confirm()
        self._confirmation.delete()

    def validate(self, data):
        """
        Validate the provided data.

        Returns:
            dict:
                The validated data.

        Raises:
            serializers.ValidationError:
                If the provided password is invalid.
        """
        user = self._confirmation.email.user

        if (
            app_settings.EMAIL_VERIFICATION_PASSWORD_REQUIRED
            and not user.check_password(data["password"])
        ):
            raise serializers.ValidationError(
                _("The provided password is invalid.")
            )

        # Add email to returned data
        data["email"] = self._confirmation.email.email

        return data

    def validate_key(self, key):
        """
        Validate the provided confirmation key.

        Returns:
            str:
                The validated confirmation key.

        Raises:
            serializers.ValidationError:
                If there is no email confirmation with the given key or
                the confirmation has expired.
        """
        try:
            confirmation = models.EmailConfirmation.objects.select_related(
                "email__user"
            ).get(key=key)
        except models.EmailConfirmation.DoesNotExist:
            raise serializers.ValidationError(
                _("The provided verification key is invalid.")
            )

        if confirmation.is_expired:
            raise serializers.ValidationError(
                _("That verification code has expired.")
            )

        # Cache confirmation instance
        self._confirmation = confirmation

        return key


class PasswordResetRequestSerializer(serializers.Serializer):
    """
    Serializer for requesting a password reset.
    """

    email = serializers.EmailField(
        help_text=_("The email address to send the password reset to.")
    )

    def save(self):
        """
        Send out a password reset if the provided data is valid.

        If the provided email address exists and is verified, a reset
        email is sent to the address.

        Returns:
            The password reset token if it was returned and ``None``
            otherwise.
        """
        try:
            email = models.EmailAddress.objects.get(
                email=self.validated_data["email"], is_verified=True
            )
        except models.EmailAddress.DoesNotExist:
            return None

        token = models.PasswordResetToken.objects.create(email=email)
        token.send()

        return token


class PasswordResetSerializer(serializers.Serializer):
    """
    Serializer for reseting a user's password.
    """

    key = serializers.UUIDField(
        help_text=_(
            "The key received by the user in the password reset " "email."
        ),
        write_only=True,
    )
    password = serializers.CharField(
        help_text=_("The user's new password."),
        style={"input_type": "password"},
        write_only=True,
    )

    def save(self):
        """
        Reset the user's password if the provided information is valid.
        """
        token = models.PasswordResetToken.objects.get(
            key=self.validated_data["key"]
        )

        token.email.user.set_password(self.validated_data["password"])
        token.email.user.save()

        logger.info("Reset password for %s", token.email.user)

        token.delete()

    def validate_key(self, key):
        """
        Validate the provided reset key.

        Returns:
            The validated key.

        Raises:
            serializers.ValidationError:
                If the provided key does not exist.
        """
        if not models.PasswordResetToken.valid_tokens.filter(key=key).exists():
            raise serializers.ValidationError(
                _("The provided reset token does not exist, or is expired.")
            )

        return key

    def validate_password(self, password):
        """
        Validate the provided password by running it through Django's
        password validation system.

        Returns:
            The validated password.

        Raises:
            ValidationError:
                If the provided password does not pass the configured
                password validators.
        """
        password_validation.validate_password(password)

        return password


class RegistrationSerializer(serializers.ModelSerializer):
    """
    Serializer for registering new users.
    """

    email = serializers.EmailField()

    class Meta(object):
        extra_kwargs = {
            "password": {
                "style": {"input_type": "password"},
                "write_only": True,
            }
        }
        fields = (get_user_model().USERNAME_FIELD, "email", "password")
        model = get_user_model()

    def create(self, validated_data):
        """
        Create a new user from the data passed to the serializer.

        If the provided email has not been verified yet, the user is
        created and a verification email is sent to the address.
        Otherwise we send a notification to the email address that
        someone attempted to register with an email that's already been
        verified.

        Args:
            validated_data (dict):
                The data passed to the serializer after it has been
                validated.

        Returns:
            A new user created from the provided data.
        """
        email = validated_data.pop("email")
        password = validated_data.pop("password")

        # We don't save the user instance yet in case the provided email
        # address already exists.
        user = get_user_model()(**validated_data)
        user.set_password(password)

        # We set an ephemeral email property so that it is included in
        # the data returned by the serializer.
        user.email = email

        email_query = models.EmailAddress.objects.filter(email=email)

        if email_query.exists():
            existing_email = email_query.get()
            existing_email.send_duplicate_notification()
        else:
            user.save()

            email_instance = models.EmailAddress.objects.create(
                email=email, is_primary=True, user=user
            )
            email_instance.send_confirmation()

            signals.user_registered.send(sender=self.__class__, user=user)

        return user

    def validate_email(self, email):
        """
        Validate the provided email address.

        Args:
            email:
                The email address to validate.

        Returns:
            The provided email address, transformed to match the RFC
            spec. Namely, the domain portion of the email must be
            lowercase.
        """
        user, domain = email.rsplit("@", 1)

        return "@".join([user, domain.lower()])

    def validate_password(self, password):
        """
        Validate the provided password.

        Args:
            password (str):
                The password provided by the user.

        Returns:
            str:
                The validated password.

        Raises:
            ValidationError:
                If the provided password doesn't pass Django's provided
                password validation.
        """
        password_validation.validate_password(password)

        return password


class ResendVerificationSerializer(serializers.Serializer):
    """
    Serializer for resending a verification email.
    """

    email = serializers.EmailField()

    def save(self):
        """
        Resend a verification email to the provided address.

        If the provided email is already verified no action is taken.
        """
        try:
            email = models.EmailAddress.objects.get(
                email=self.validated_data["email"], is_verified=False
            )

            logger.debug(
                "Resending verification email to %s",
                self.validated_data["email"],
            )

            email.send_confirmation()
        except models.EmailAddress.DoesNotExist:
            logger.debug(
                "Not resending verification email to %s because the address "
                "doesn't exist in the database.",
                self.validated_data["email"],
            )



import datetime

try:
    from unittest import mock
except ImportError:
    import mock

from rest_email_auth import models


def test_confirm(
    email_confirmation_factory, email_factory, email_verification_listener
):
    """
    Confirming a confirmation should mark the associated email address
    as verified.
    """
    email = email_factory()
    confirmation = email_confirmation_factory(email=email)

    confirmation.confirm()
    email.refresh_from_db()

    assert email.is_verified
    assert email_verification_listener.call_count == 1
    assert (
        email_verification_listener.call_args[1]["sender"]
        == confirmation.__class__
    )
    assert email_verification_listener.call_args[1]["email"] == email


def test_create_email_confirmation(email_factory):
    """
    Test creating a new email confirmation.
    """
    models.EmailConfirmation.objects.create(email=email_factory())


def test_is_expired(email_confirmation_factory):
    """
    If it has been more than a day since the confirmation was created it
    should be marked as expired.
    """
    confirmation = email_confirmation_factory()
    future_time = confirmation.created_at + datetime.timedelta(days=2)

    with mock.patch(
        "rest_email_auth.models.timezone.now",
        autospec=True,
        return_value=future_time,
    ):
        assert confirmation.is_expired


def test_is_expired_unexpired(email_confirmation_factory):
    """
    If it has been less than a day since the confirmation was created it
    should not be marked as expired.
    """
    confirmation = email_confirmation_factory()

    assert not confirmation.is_expired


@mock.patch("rest_email_auth.models.email_utils.send_email")
def test_send(mock_send_email, email_confirmation_factory, settings):
    """
    Sending the confirmation should send an email to the associated
    email address.
    """
    settings.REST_EMAIL_AUTH = {
        "EMAIL_VERIFICATION_URL": "https://example.com/verify?key={key}"
    }

    confirmation = email_confirmation_factory()
    confirmation.send()

    url = settings.REST_EMAIL_AUTH["EMAIL_VERIFICATION_URL"].format(
        key=confirmation.key
    )

    context = {"verification_url": url}

    assert mock_send_email.call_count == 1
    assert mock_send_email.call_args[1] == {
        "context": context,
        "from_email": settings.DEFAULT_FROM_EMAIL,
        "recipient_list": [confirmation.email.email],
        "subject": "Please Verify Your Email Address",
        "template_name": "rest_email_auth/emails/verify-email",
    }



import datetime

try:
    from unittest import mock
except ImportError:
    import mock

from rest_email_auth import models


def test_create(email_factory):
    """
    Test creating a new password reset token.
    """
    models.PasswordResetToken.objects.create(email=email_factory())


def test_string_conversion(password_reset_token_factory):
    """
    Converting a password reset token to a string should return
    information about the token's owner.
    """
    token = password_reset_token_factory()
    expected = "Password reset token for '{}'".format(token.email.user)

    assert str(token) == expected


@mock.patch("rest_email_auth.models.email_utils.send_email")
def test_send(mock_send_email, password_reset_token_factory, settings):
    """
    Sending the token should send an email to the address associated
    with the token.
    """
    settings.REST_EMAIL_AUTH = {
        "PASSWORD_RESET_URL": "https://example.com/reset/{key}"
    }

    token = password_reset_token_factory()
    token.send()

    url = settings.REST_EMAIL_AUTH["PASSWORD_RESET_URL"].format(key=token.key)

    context = {"reset_url": url}

    assert mock_send_email.call_count == 1
    assert mock_send_email.call_args[1] == {
        "context": context,
        "from_email": settings.DEFAULT_FROM_EMAIL,
        "recipient_list": [token.email.email],
        "subject": "Reset Your Password",
        "template_name": "rest_email_auth/emails/reset-password",
    }


def test_valid_tokens(password_reset_token_factory, app_settings):
    """
    The valid tokens queryset should include unexpired tokens.
    """
    app_settings.PASSWORD_RESET_EXPIRATION = datetime.timedelta(hours=1)

    token = password_reset_token_factory()

    assert list(models.PasswordResetToken.valid_tokens.all()) == [token]


def test_valid_tokens_expired(password_reset_token_factory, app_settings):
    """
    The valid tokens queryset should not include expired tokens.
    """
    app_settings.PASSWORD_RESET_EXPIRATION = datetime.timedelta()

    password_reset_token_factory()

    assert not models.PasswordResetToken.valid_tokens.exists()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-31 20:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("rest_email_auth", "0003_unique_emails")]

    operations = [
        migrations.AddField(
            model_name="emailaddress",
            name="is_primary",
            field=models.BooleanField(
                default=False,
                help_text="Boolean indicating if the email is the user's primary address.",
                verbose_name="is primary",
            ),
        )
    ]



"""Admin configurations for the ``rest_email_auth`` app.
"""

from django.contrib import admin
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from rest_email_auth import models


@admin.register(models.EmailAddress)
class EmailAddressAdmin(admin.ModelAdmin):
    """
    Admin for ``EmailAddress`` instances.
    """

    fields = ("user", "email", "is_verified", "created_at")
    list_display = ("email", "user", "is_verified", "created_at")
    list_filter = ("is_verified",)
    readonly_fields = ("created_at",)
    search_fields = ("email",)


@admin.register(models.EmailConfirmation)
class EmailConfirmationAdmin(admin.ModelAdmin):
    """
    Admin for ``EmailConfirmation`` instances.
    """

    fields = ("email", "key", "created_at")
    list_display = ("email", "created_at")
    readonly_fields = ("email", "key", "created_at")
    search_fields = ("email__email",)


@admin.register(models.PasswordResetToken)
class PasswordResetTokenAdmin(admin.ModelAdmin):
    """
    Admin for ``PasswordResetToken`` instances.
    """

    fields = ("email", "key", "created_at")
    list_display = ("get_user", "email", "created_at")
    readonly_fields = fields
    search_fields = (get_user_model().USERNAME_FIELD, "email__email")

    def get_user(self, obj):
        """
        Get the user that owns the password reset token.
        """
        return obj.email.user

    get_user.admin_order_field = "email__user"
    get_user.short_description = _("user")



import datetime
import logging
import uuid

from django.conf import settings
from django.db import models, transaction
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _

import email_utils

from rest_email_auth import app_settings, managers, signals


logger = logging.getLogger(__name__)


def generate_token():
    """
    Get a random 64 character string.

    Returns:
        str:
            A random 64 character string.
    """
    return get_random_string(length=64)


class EmailAddress(models.Model):
    """
    A user's email address.
    """

    created_at = models.DateTimeField(
        auto_now_add=True, verbose_name=_("created at")
    )
    email = models.EmailField(
        max_length=255, unique=True, verbose_name=_("email")
    )
    is_primary = models.BooleanField(
        default=False,
        help_text=_(
            "Boolean indicating if the email is the user's primary " "address."
        ),
        verbose_name=_("is primary"),
    )
    is_verified = models.BooleanField(
        default=False,
        help_text=_(
            "Boolean indicating if the user has verified ownership of "
            "the email address."
        ),
        verbose_name=_("is verified"),
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="email_addresses",
        related_query_name="email_address",
        verbose_name=_("user"),
    )

    # Use custom manager
    objects = managers.EmailAddressManager()

    class Meta(object):
        verbose_name = _("email address")
        verbose_name_plural = _("email addresses")

    def __str__(self):
        """
        Get a string representation of the email address.

        Returns:
            str:
                A text version of the email address. Ex:
                ``test@example.com``.
        """
        return self.email

    def send_confirmation(self):
        """
        Send a verification email for the email address.
        """
        confirmation = EmailConfirmation.objects.create(email=self)
        confirmation.send()

    def send_duplicate_notification(self):
        """
        Send a notification about a duplicate signup.
        """
        email_utils.send_email(
            from_email=settings.DEFAULT_FROM_EMAIL,
            recipient_list=[self.email],
            subject=_("Registration Attempt"),
            template_name="rest_email_auth/emails/duplicate-email",
        )

        logger.info("Sent duplicate email notification to: %s", self.email)

    def set_primary(self):
        """
        Set this email address as the user's primary email.
        """
        query = EmailAddress.objects.filter(is_primary=True, user=self.user)
        query = query.exclude(pk=self.pk)

        # The transaction is atomic so there is never a gap where a user
        # has no primary email address.
        with transaction.atomic():
            query.update(is_primary=False)

            self.is_primary = True
            self.save()

        logger.info(
            "Set %s as the primary email address for %s.",
            self.email,
            self.user,
        )


class EmailConfirmation(models.Model):
    """
    Model to store a token used to verify an email address.
    """

    created_at = models.DateTimeField(
        auto_now_add=True, verbose_name=_("created at")
    )
    email = models.ForeignKey(
        "rest_email_auth.EmailAddress",
        on_delete=models.CASCADE,
        related_name="confirmations",
        related_query_name="confirmation",
        verbose_name=_("email"),
    )
    key = models.CharField(
        default=generate_token,
        editable=False,
        max_length=255,
        verbose_name=_("confirmation key"),
    )

    class Meta(object):
        verbose_name = _("email confirmation")
        verbose_name_plural = _("email confirmations")

    def confirm(self):
        """
        Mark the instance's email as verified.
        """
        self.email.is_verified = True
        self.email.save()

        signals.email_verified.send(email=self.email, sender=self.__class__)

        logger.info("Verified email address: %s", self.email.email)

    @property
    def is_expired(self):
        """
        Determine if the confirmation has expired.

        Returns:
            bool:
                ``True`` if the confirmation has expired and ``False``
                otherwise.
        """
        expiration_time = self.created_at + datetime.timedelta(days=1)

        return timezone.now() > expiration_time

    def send(self):
        """
        Send a verification email to the user.
        """
        context = {
            "verification_url": app_settings.EMAIL_VERIFICATION_URL.format(
                key=self.key
            )
        }

        email_utils.send_email(
            context=context,
            from_email=settings.DEFAULT_FROM_EMAIL,
            recipient_list=[self.email.email],
            subject=_("Please Verify Your Email Address"),
            template_name="rest_email_auth/emails/verify-email",
        )

        logger.info(
            "Sent confirmation email to %s for user #%d",
            self.email.email,
            self.email.user.id,
        )


class PasswordResetToken(models.Model):
    """
    Store a token that can be used to reset a user's password.
    """

    created_at = models.DateTimeField(
        auto_now_add=True,
        help_text=_("The time at which the password reset token was created."),
        verbose_name=_("created at"),
    )
    email = models.ForeignKey(
        "rest_email_auth.EmailAddress",
        editable=False,
        help_text=_("The email address used to request the password reset."),
        on_delete=models.CASCADE,
        related_name="password_reset_tokens",
        related_query_name="password_reset_token",
        verbose_name=_("email address"),
    )
    key = models.UUIDField(
        default=uuid.uuid4,
        editable=False,
        help_text=_(
            "The token that authorizes the user to reset their " "password."
        ),
        verbose_name=_("key"),
    )

    # Custom managers. We have to explicitly define the default manager
    # so that it doesn't get overwritten.
    objects = models.Manager()
    valid_tokens = managers.ValidPasswordResetTokenManager()

    class Meta:
        verbose_name = _("password reset token")
        verbose_name_plural = _("password reset tokens")

    def __str__(self):
        """
        Get a string representation of the instance.

        Returns:
            Information about the token's owner.
        """
        return "Password reset token for '{}'".format(self.email.user)

    def send(self):
        """
        Send the password reset token to the user.
        """
        context = {
            "reset_url": app_settings.PASSWORD_RESET_URL.format(key=self.key)
        }

        email_utils.send_email(
            context=context,
            from_email=settings.DEFAULT_FROM_EMAIL,
            recipient_list=[self.email.email],
            subject=_("Reset Your Password"),
            template_name="rest_email_auth/emails/reset-password",
        )

        logger.info("Sent password reset email to %s", self.email)
